@@ -20,9 +20,10 @@ import sys
 import re
 from copy import copy
 
+import six
+
 import astroid
 from astroid import modutils
-
 from pylint.interfaces import IAstroidChecker, INFERENCE, INFERENCE_FAILURE, HIGH
 from pylint.utils import get_global_option
 from pylint.checkers import BaseChecker
@@ -30,8 +31,9 @@ from pylint.checkers.utils import (
     PYMETHODS, is_ancestor_name, is_builtin,
     is_defined_before, is_error, is_func_default, is_func_decorator,
     assign_parent, check_messages, is_inside_except, clobber_in_except,
-    get_all_elements, has_known_bases, node_ignores_exception)
-import six
+    get_all_elements, has_known_bases, node_ignores_exception,
+    is_inside_abstract_class, is_comprehension, is_iterable,
+    safe_infer)
 
 SPECIAL_OBJ = re.compile("^_{2}[a-z]+_{2}$")
 FUTURE = '__future__'
@@ -248,7 +250,7 @@ MSGS = {
     'W0614': ('Unused import %s from wildcard import',
               'unused-wildcard-import',
               'Used when an imported module or variable is not used from a \
-              \'from X import *\' style import.'),
+              `\'from X import *\'` style import.'),
 
     'W0621': ('Redefining name %r from outer scope (line %s)',
               'redefined-outer-name',
@@ -578,20 +580,21 @@ builtins. Remember that you should avoid to define new builtins when possible.'
                 continue
             if isinstance(stmt, (astroid.Import, astroid.ImportFrom)):
                 # Detect imports, assigned to global statements.
-                if global_names:
-                    skip = False
-                    for import_name, import_alias in stmt.names:
-                        # If the import uses an alias, check only that.
-                        # Otherwise, check only the import name.
-                        if import_alias:
-                            if import_alias in global_names:
-                                skip = True
-                                break
-                        elif import_name in global_names:
+                if not global_names:
+                    continue
+                skip = False
+                for import_name, import_alias in stmt.names:
+                    # If the import uses an alias, check only that.
+                    # Otherwise, check only the import name.
+                    if import_alias:
+                        if import_alias in global_names:
                             skip = True
                             break
-                    if skip:
-                        continue
+                    elif import_name in global_names:
+                        skip = True
+                        break
+                if skip:
+                    continue
 
             # care about functions with unknown argument (builtins)
             if name in argnames:
@@ -619,6 +622,9 @@ builtins. Remember that you should avoid to define new builtins when possible.'
                         continue
                 self.add_message('unused-variable', args=name, node=stmt)
 
+    visit_asyncfunctiondef = visit_functiondef
+    leave_asyncfunctiondef = leave_functiondef
+
     @check_messages('global-variable-undefined', 'global-variable-not-assigned',
                     'global-statement', 'global-at-module-level',
                     'redefined-builtin')
@@ -726,7 +732,7 @@ builtins. Remember that you should avoid to define new builtins when possible.'
             assign = astmts[0].assign_type()
             if (isinstance(assign, (astroid.For, astroid.Comprehension,
                                     astroid.GeneratorExp))
-                    and not assign.statement() is node.statement()):
+                    and assign.statement() is not node.statement()):
                 self.add_message('undefined-loop-variable', args=name, node=node)
 
     @check_messages('redefine-in-handler')
@@ -886,6 +892,7 @@ builtins. Remember that you should avoid to define new builtins when possible.'
             start_index = len(self._to_consume) - 1
         # iterates through parent scopes, from the inner to the outer
         base_scope_type = self._to_consume[start_index][-1]
+        # pylint: disable=too-many-nested-blocks; refactoring this block is a pain.
         for i in range(start_index, -1, -1):
             to_consume, consumed, scope_type = self._to_consume[i]
             # if the current scope is a class scope but it's not the inner
@@ -957,11 +964,11 @@ builtins. Remember that you should avoid to define new builtins when possible.'
                         #      f = 42
                         if isinstance(frame, astroid.ClassDef) and name in frame.locals:
                             if isinstance(node.parent, astroid.Arguments):
-                                # Doing the following is fine:
-                                #   class A:
-                                #      x = 42
-                                #      y = lambda attr=x: attr
                                 if stmt.fromlineno <= defstmt.fromlineno:
+                                    # Doing the following is fine:
+                                    #   class A:
+                                    #      x = 42
+                                    #      y = lambda attr=x: attr
                                     self.add_message('used-before-assignment',
                                                      args=name, node=node)
                             else:
@@ -1030,7 +1037,8 @@ builtins. Remember that you should avoid to define new builtins when possible.'
 
         targets = node.targets[0].itered()
         try:
-            for infered in node.value.infer():
+            infered = safe_infer(node.value)
+            if infered is not None:
                 self._check_unpacking(infered, node, targets)
         except astroid.InferenceError:
             return
@@ -1039,6 +1047,10 @@ builtins. Remember that you should avoid to define new builtins when possible.'
         """ Check for unbalanced tuple unpacking
         and unpacking non sequences.
         """
+        if is_inside_abstract_class(node):
+            return
+        if is_comprehension(node):
+            return
         if infered is astroid.YES:
             return
         if (isinstance(infered.parent, astroid.Arguments) and
@@ -1059,19 +1071,10 @@ builtins. Remember that you should avoid to define new builtins when possible.'
                                        len(targets),
                                        len(values)))
         # attempt to check unpacking may be possible (ie RHS is iterable)
-        elif isinstance(infered, astroid.Instance):
-            for meth in ('__iter__', '__getitem__'):
-                try:
-                    infered.getattr(meth)
-                    break
-                except astroid.NotFoundError:
-                    continue
-            else:
+        else:
+            if not is_iterable(infered):
                 self.add_message('unpacking-non-sequence', node=node,
                                  args=(_get_unpacking_extra_info(node, infered),))
-        else:
-            self.add_message('unpacking-non-sequence', node=node,
-                             args=(_get_unpacking_extra_info(node, infered),))
 
 
     def _check_module_attrs(self, node, module, module_names):
